@@ -7,9 +7,9 @@ from urlparse import urlparse
 from functools import wraps
 
 from .things import Blob, ListBlob, Account, identify_thing
-from .util import relative_url, pull_data_dict, html_unicode_unescape
-from .exceptions import NotLoggedIn, BadResponse, PostError, LoginFail
-from .const import DEFAULT_USER_AGENT, LOGIN_URL, POST_ERROR_PATTERN, API_PERIOD, SUBMIT_RESPONSE_LINK_PATTERN
+from .util import reddit_url, html_unicode_unescape, assert_truthy
+from .exceptions import NotLoggedIn, BadResponse, PostError, LoginFail, UnexpectedResponse
+from .const import DEFAULT_USER_AGENT, LOGIN_URL, API_PERIOD
 
 
 def _limit_rate(f, period=API_PERIOD):
@@ -83,6 +83,17 @@ class Reddit(object):
             kwargs.setdefault('headers', {'User-Agent': self._user_agent})
         return kwargs
     
+    def _inject_post_data(self, kwargs):
+        if 'data' in kwargs:
+            data = kwargs['data'].copy()
+        else:
+            data = dict()
+        data.setdefault('api_type', 'json')
+        if self.logged_in:
+            data.setdefault('uh', self._modhash)
+        kwargs['data'] = data
+        return kwargs
+    
     def _thingify(self, obj, path=None):
         def helper(obj_, dict_):
             for k, v in dict_.items():
@@ -122,7 +133,7 @@ class Reddit(object):
         :param \*\*kwargs: extra keyword arguments to be passed to :meth:`requests.get`
         """
         kwargs = self._inject_request_kwargs(kwargs)
-        url = relative_url(*args)
+        url = reddit_url(*args)
         r = requests.get(url, **kwargs)
         # print r.url
         if r.status_code == 200:
@@ -135,32 +146,35 @@ class Reddit(object):
     def post(self, *args, **kwargs):
         """Sends a POST request to a reddit path determined by ``args``.  Basically ``.post('foo', 'bar', 'baz')`` will POST http://www.reddit.com/foo/bar/baz/.json.  ``kwargs`` supplied will be passed to ``requests.post`` after having ``modhash`` and ``cookies`` injected, and after having modhash injected into ``kwargs['data']`` if logged in.  Injection only occurs if they don't already exist.
         
-        Returns :class:`requests.Response` object, raises :class:`exceptions.BadResponse` if not a 200 Response, or raises :class:`exceptions.POST_ERROR` if a reddit error was returned.
+        Returns received response JSON content as a dict.
+        
+        Raises :class:`exceptions.BadResponse` if not a 200 response or no JSON content received or raises :class:`exceptions.PostError` if a reddit error was returned.
         
         :param \*args: strings that will form the path to POST
         :param \*\*kwargs: extra keyword arguments to be passed to ``requests.POST``
         """
         kwargs = self._inject_request_kwargs(kwargs)
-        if self._modhash:
-            if 'data' in kwargs:
-                data = kwargs['data'].copy()
-                data.setdefault('uh', self._modhash)
-                kwargs['data'] = data
-            else:
-                kwargs['data'] = dict(uh=self._modhash)
-        url = relative_url(*args)
+        kwargs = self._inject_post_data(kwargs)
+        url = reddit_url(*args)
         r = requests.post(url, **kwargs)
         if r.status_code == 200:
-            errors = POST_ERROR_PATTERN.findall(r.content)
+            try:
+                j = json.loads(r.content)
+            except ValueError:
+                raise BadResponse(r)
+            try:
+                errors = j['json']['errors']
+            except Exception:
+                errors = None
             if errors:
                 raise PostError(errors)
             else:
-                return r
+                return j
         else:
             raise BadResponse(r)
 
     def login(self, username, password):
-        """Logs into reddit with supplied credentials using SSL.  Returns :class:`requests.Response` object, or raises :class:`exceptions.LoginFail` or :class:`exceptions.BadResponse`.
+        """Logs into reddit with supplied credentials using SSL.  Returns :class:`requests.Response` object, or raises :class:`exceptions.LoginFail` or :class:`exceptions.BadResponse` if not a 200 response.
         
         URL: ``https://ssl.reddit.com/api/login``
         
@@ -196,6 +210,15 @@ class Reddit(object):
             args = ('r', sr) + args
         return self._limit_get(*args, limit=limit)
     
+    def by_id(self, id_):
+        """GETs a link by ID.  Returns :class:`things.Link` object.
+        
+        URL: ``http://www.reddit.com/by_id/<id_>``
+        
+        :param id\_: full name of link
+        """
+        return self.get('by_id', id_)[0]
+    
     def hot(self, sr=None, limit=None):
         """GETs hot links.  If ``sr`` is ``None``, gets from main.  Returns :class:`things.Listing` object.
         
@@ -204,7 +227,7 @@ class Reddit(object):
         :param sr: subreddit name
         :param limit: max number of submissions to get
         """
-        return self._subreddit_get(sr, None, limit)
+        return self._subreddit_get(sr, None, limit=limit)
     
     def new(self, sr=None, limit=None):
         """GETs new links.  If ``sr`` is ``None``, gets from main.  Returns :class:`things.Listing` object.
@@ -214,7 +237,7 @@ class Reddit(object):
         :param sr: subreddit name
         :param limit: max number of submissions to get
         """
-        return self._subreddit_get(sr, 'new', limit)
+        return self._subreddit_get(sr, 'new', limit=limit)
     
     def top(self, sr=None, limit=None):
         """GETs top links.  If ``sr`` is ``None``, gets from main.  Returns :class:`things.Listing` object.
@@ -224,7 +247,7 @@ class Reddit(object):
         :param sr: subreddit name
         :param limit: max number of submissions to get
         """
-        return self._subreddit_get(sr, 'top', limit)
+        return self._subreddit_get(sr, 'top', limit=limit)
     
     def controversial(self, sr=None, limit=None):
         """GETs controversial links.  If ``sr`` is ``None``, gets from main.  Returns :class:`things.Listing` object.
@@ -234,7 +257,7 @@ class Reddit(object):
         :param sr: subreddit name
         :param limit: max number of submissions to get
         """
-        return self._subreddit_get(sr, 'controversial', limit)
+        return self._subreddit_get(sr, 'controversial', limit=limit)
     
     def comments(self, sr=None, limit=None):
         """GETs newest comments.  If ``sr`` is ``None``, gets all.  Returns :class:`things.Listing` object.
@@ -244,14 +267,14 @@ class Reddit(object):
         :param sr: subreddit name
         :param limit: max number of comments to get
         """
-        return self._subreddit_get(sr, 'comments', limit)
+        return self._subreddit_get(sr, 'comments', limit=limit)
     
     def user(self, username):
         """GETs user info.  Returns :class:`things.Account` object.
         
         URL: ``http://www.reddit.com/user/<username>/about/``
         
-        :param username: username of user to get info
+        :param username: username of user
         """
         return self.get('user', username, 'about')
     
@@ -318,7 +341,7 @@ class Reddit(object):
         """
         return self._limit_get('user', user, 'submitted', limit=limit)
     
-    def moderators(self, sr):
+    def moderators(self, sr, limit=None):
         """GETs moderators of subreddit ``sr``.  Returns :class:`things.ListBlob` object.
         
         **NOTE**: The :class:`things.Account` objects in the returned ListBlob *only* have ``id`` and ``name`` set.  This is because that's all reddit returns.  If you need full info on each moderator, you must individually GET them using :meth:`user` or :meth:`things.Account.about`.
@@ -327,7 +350,7 @@ class Reddit(object):
         
         :param sr: name of subreddit
         """
-        userlist = self.get('r', sr, 'about', 'moderators')
+        userlist = self._limit_get('r', sr, 'about', 'moderators', limit=limit)
         return _process_userlist(userlist)
     
     @_login_required
@@ -341,16 +364,17 @@ class Reddit(object):
         return self.get('api', 'me')
     
     @_login_required
-    def mine(self, limit=None):
-        """Login required.  GETs logged in user's subscribed subreddits.  Returns :class:`things.Listing` object.
+    def mine(self, which='subscriber', limit=None):
+        """Login required.  GETs logged in user's subreddits.  Returns :class:`things.Listing` object.
         
         See https://github.com/reddit/reddit/wiki/API%3A-mine.json.
         
-        URL: ``http://www.reddit.com/reddits/mine?limit=<limit>``
+        URL: ``http://www.reddit.com/reddits/mine[/(subscriber|contributor|moderator)]?limit=<limit>``
         
+        :param which: 'subscriber', 'contributor', or 'moderator'
         :param limit: max number of subreddits to get
         """
-        return self._limit_get('reddits', 'mine', limit=limit)
+        return self._limit_get('reddits', 'mine', which, limit=limit)
     
     @_login_required
     def saved(self, limit=None):
@@ -364,7 +388,7 @@ class Reddit(object):
     
     @_login_required
     def vote(self, id_, dir_):
-        """Login required.  POSTs a vote.  Returns :class:`requests.Response` object.
+        """Login required.  POSTs a vote.  Returns True or raises :class:`exceptions.UnexpectedResponse` if non-"truthy" value in response.
         
         See https://github.com/reddit/reddit/wiki/API%3A-vote.
         
@@ -374,11 +398,12 @@ class Reddit(object):
         :param dir\_: direction of vote (1, 0, or -1)
         """
         data = dict(id=id_, dir=dir_)
-        return self.post('api', 'vote', data=data)
+        j = self.post('api', 'vote', data=data)
+        return assert_truthy(j)
     
     @_login_required
     def upvote(self, id_):
-        """Login required.  POSTs an upvote (1).  Returns :class:`requests.Response` object.
+        """Login required.  POSTs an upvote (1).  Returns True or raises :class:`exceptions.UnexpectedResponse` if non-"truthy" value in response.
         
         See https://github.com/reddit/reddit/wiki/API%3A-vote.
         
@@ -390,7 +415,7 @@ class Reddit(object):
     
     @_login_required
     def downvote(self, id_):
-        """Login required.  POSTs a downvote (-1).  Returns :class:`requests.Response` object.
+        """Login required.  POSTs a downvote (-1).  Returns True or raises :class:`exceptions.UnexpectedResponse` if non-"truthy" value in response.
         
         See https://github.com/reddit/reddit/wiki/API%3A-vote.
         
@@ -402,7 +427,7 @@ class Reddit(object):
     
     @_login_required
     def unvote(self, id_):
-        """Login required.  POSTs a null vote (0).  Returns :class:`requests.Response` object.
+        """Login required.  POSTs a null vote (0).  Returns True or raises :class:`exceptions.UnexpectedResponse` if non-"truthy" value in response.
         
         See https://github.com/reddit/reddit/wiki/API%3A-vote.
         
@@ -424,17 +449,15 @@ class Reddit(object):
         :param text: comment text
         """
         data = dict(parent=parent, text=text)
-        r = self.post('api', 'comment', data=data)
+        j = self.post('api', 'comment', data=data)
         try:
-            j = json.loads(r.content)
-            data_dict = pull_data_dict(j['jquery'])
-            return self._thingify(data_dict)
+            return self._thingify(j['json']['data']['things'][0])
         except Exception:
-            raise BadResponse(r)
+            raise UnexpectedResponse(j)
     
     @_login_required
     def edit(self, id_, text):
-        """Login required.  Sends POST to change selftext or comment text to ``text``.  Returns :class:`things.Comment` or :class:`things.Link` object depending on what's being edited.
+        """Login required.  Sends POST to change selftext or comment text to ``text``.  Returns :class:`things.Comment` or :class:`things.Link` object depending on what's being edited.  Raises :class:`UnexpectedResponse` if neither is returned.
         
         URL: ``http://www.reddit.com/api/editusertext/``
         
@@ -442,13 +465,11 @@ class Reddit(object):
         :param text: new self or comment text
         """
         data = dict(thing_id=id_, text=text)
-        r = self.post('api', 'editusertext', data=data)
+        j = self.post('api', 'editusertext', data=data)
         try:
-            j = json.loads(r.content)
-            data_dict = pull_data_dict(j['jquery'])
-            return self._thingify(data_dict)
+            return self._thingify(j['json']['data']['things'][0])
         except Exception:
-            raise BadResponse(r)
+            raise UnexpectedResponse(j)
     
     @_login_required
     def _submit(self, sr, title, kind, url=None, text=None, follow=True):
@@ -457,17 +478,17 @@ class Reddit(object):
             data['url'] = url
         elif kind == 'self':
             data['text'] = text
-        r = self.post('api', 'submit', data=data)
+        j = self.post('api', 'submit', data=data)
         try:
-            m = SUBMIT_RESPONSE_LINK_PATTERN.search(r.content)
+            path = urlparse(j['json']['data']['url']).path
             if follow:
-                r2 = self.get(urlparse(m.group(1)).path)
-                link = r2[0][0]
+                r = self.get(path)
+                link = r[0][0]
                 return link
             else:
-                return m.group(1)
+                return path
         except Exception:
-            raise BadResponse(r)
+            raise UnexpectedResponse(r)
     
     @_login_required
     def submit_link(self, sr, title, url, follow=True):
@@ -507,18 +528,19 @@ class Reddit(object):
     
     @_login_required
     def delete(self, id_):
-        """Login required.  Send POST to delete an object.  Returns :class:`requests.Response` object.
+        """Login required.  Send POST to delete an object.  Returns True or raises :class:`exceptions.UnexpectedResponse` if non-"truthy" value in response.
         
         URL: ``http://www.reddit.com/api/del/``
         
-        :param sr: full id of object to delete   
+        :param id\_: full id of object to delete   
         """
         data = dict(id=id_)
-        return self.post('api', 'del', data=data)
+        j = self.post('api', 'del', data=data)
+        return assert_truthy(j)
     
     @_login_required
     def save(self, id_):
-        """Login required.  Sends POST to save a link.  Returns :class:`requests.Response` object.
+        """Login required.  Sends POST to save a link.  Returns True or raises :class:`exceptions.UnexpectedResponse` if non-"truthy" value in response.
         
         See https://github.com/reddit/reddit/wiki/API%3A-save.
         
@@ -527,11 +549,12 @@ class Reddit(object):
         :param id\_: full id of link to save
         """
         data = dict(id=id_)
-        return self.post('api', 'save', data=data)
+        j = self.post('api', 'save', data=data)
+        return assert_truthy(j)
     
     @_login_required
     def unsave(self, id_):
-        """Login required.  Sends POST to unsave a link.  Returns :class:`requests.Response` object.
+        """Login required.  Sends POST to unsave a link.  Returns True or raises :class:`exceptions.UnexpectedResponse` if non-"truthy" value in response.
         
         See https://github.com/reddit/reddit/wiki/API%3A-unsave.
         
@@ -540,11 +563,12 @@ class Reddit(object):
         :param id\_: full id of link to unsave
         """
         data = dict(id=id_)
-        return self.post('api', 'unsave', data=data)
+        j = self.post('api', 'unsave', data=data)
+        return assert_truthy(j)
     
     @_login_required
     def hide(self, id_):
-        """Login required.  Sends POST to hide a link.  Returns :class:`requests.Response` object.
+        """Login required.  Sends POST to hide a link.  Returns True or raises :class:`exceptions.UnexpectedResponse` if non-"truthy" value in response.
         
         See https://github.com/reddit/reddit/wiki/API%3A-hide.
         
@@ -553,11 +577,12 @@ class Reddit(object):
         :param id\_: full id of link to hide
         """
         data = dict(id=id_)
-        return self.post('api', 'hide', data=data)
+        j = self.post('api', 'hide', data=data)
+        return assert_truthy(j)
     
     @_login_required
     def unhide(self, id_):
-        """Login required.  Sends POST to unhide a link.  Returns :class:`requests.Response` object.
+        """Login required.  Sends POST to unhide a link.  Returns True or raises :class:`exceptions.UnexpectedResponse` if non-"truthy" value in response.
         
         See https://github.com/reddit/reddit/wiki/API%3A-unhide.
         
@@ -566,42 +591,46 @@ class Reddit(object):
         :param id\_: full id of link to unhide
         """
         data = dict(id=id_)
-        return self.post('api', 'unhide', data=data)
+        j = self.post('api', 'unhide', data=data)
+        return assert_truthy(j)
     
     @_login_required
     def marknsfw(self, id_):
-        """Login required.  Sends POST to mark link as NSFW.  Returns :class:`requests.Response` object.
+        """Login required.  Sends POST to mark link as NSFW.  Returns True or raises :class:`exceptions.UnexpectedResponse` if non-"truthy" value in response.
         
         URL: ``http://www.reddit.com/api/marknsfw/``
         
         :param id\_: full id of link to mark
         """
         data = dict(id=id_)
-        return self.post('api', 'marknsfw', data=data)
+        j = self.post('api', 'marknsfw', data=data)
+        return assert_truthy(j)
     
     @_login_required
     def unmarknsfw(self, id_):
-        """Login required.  Sends POST to unmark link as NSFW.  Returns :class:`requests.Response` object.
+        """Login required.  Sends POST to unmark link as NSFW.  Returns True or raises :class:`exceptions.UnexpectedResponse` if non-"truthy" value in response.
         
         URL: ``http://www.reddit.com/api/unmarknsfw/``
         
         :param id\_: full id of link to unmark
         """
         data = dict(id=id_)
-        return self.post('api', 'unmarknsfw', data=data)
+        j = self.post('api', 'unmarknsfw', data=data)
+        return assert_truthy(j)
     
     @_login_required
     def report(self, id_):
-        """Login required.  Sends POST to report a link.  Returns :class:`requests.Response` object.
+        """Login required.  Sends POST to report a link.  Returns True or raises :class:`exceptions.UnexpectedResponse` if non-"truthy" value in response.
         
         URL: ``http://www.reddit.com/api/report/``
         
         :param id\_: full id of link to report
         """
         data = dict(id=id_)
-        return self.post('api', 'report', data=data)
+        j = self.post('api', 'report', data=data)
+        return assert_truthy(j)
     
-    # reddit seems to block bots from sharing, so this doesnt work :(
+    # reddit seems to block bots from sharing, so this doesnt work
     @_login_required
     def share(self, parent, share_from, replyto, share_to, message):
         data = dict(parent=parent, share_from=share_from, replyto=replyto, share_to=share_to, message=message)
@@ -614,7 +643,7 @@ class Reddit(object):
     
     @_login_required
     def compose(self, to, subject, text):
-        """Login required.  Sends POST to send a message to a user.  Returns :class:`requests.Response` object.
+        """Login required.  Sends POST to send a message to a user.  Returns True or raises :class:`exceptions.UnexpectedResponse` if non-"truthy" value in response.
         
         URL: ``http://www.reddit.com/api/compose/``
         
@@ -625,55 +654,36 @@ class Reddit(object):
         if isinstance(to, Account):
             to = to.name
         data = dict(to=to, subject=subject, text=text)
-        return self.post('api', 'compose', data=data)
+        j = self.post('api', 'compose', data=data)
+        return assert_truthy(j)
     
     @_login_required
     def read_message(self, id_):
-        """Login required.  Send POST to mark a message as read.  Returns :class:`requests.Response` object.
+        """Login required.  Send POST to mark a message as read.  Returns True or raises :class:`exceptions.UnexpectedResponse` if non-"truthy" value in response.
         
         URL: ``http://www.reddit.com/api/read_message/``
         
         :param id\_: full id of message to mark
         """
         data = dict(id=id_)
-        return self.post('api', 'read_message', data=data)
+        j = self.post('api', 'read_message', data=data)
+        return assert_truthy(j)
     
     @_login_required
     def unread_message(self, id_):
-        """Login required.  Send POST to unmark a message as read.  Returns :class:`requests.Response` object.
+        """Login required.  Send POST to unmark a message as read.  Returns True or raises :class:`exceptions.UnexpectedResponse` if non-"truthy" value in response.
         
         URL: ``http://www.reddit.com/api/unread_message/``
         
         :param id\_: full id of message to unmark
         """
         data = dict(id=id_)
-        return self.post('api', 'unread_message', data=data)
-    
-    @_login_required
-    def hide_message(self, id_):
-        """Login required.  Send POST to hide a message.  Returns :class:`requests.Response` object.
-        
-        URL: ``http://www.reddit.com/api/hide_message/``
-        
-        :param id\_: full id of message to hide
-        """
-        data = dict(id=id_)
-        return self.post('api', 'hide_message', data=data)
-    
-    @_login_required
-    def unhide_message(self, id_):
-        """Login required.  Send POST to unhide a message.  Returns :class:`requests.Response` object.
-        
-        URL: ``http://www.reddit.com/api/unhide_message/``
-        
-        :param id\_: full id of message to unhide
-        """
-        data = dict(id=id_)
-        return self.post('api', 'unhide_message', data=data)
+        j = self.post('api', 'unread_message', data=data)
+        return assert_truthy(j)
     
     @_login_required
     def subscribe(self, sr):
-        """Login required.  Send POST to subscribe to a subreddit.  If ``sr`` is the name of the subreddit, a GET request is sent to retrieve the full id of the subreddit, which is necessary for this API call.  Returns :class:`requests.Response` object.
+        """Login required.  Send POST to subscribe to a subreddit.  If ``sr`` is the name of the subreddit, a GET request is sent to retrieve the full id of the subreddit, which is necessary for this API call.  Returns True or raises :class:`exceptions.UnexpectedResponse` if non-"truthy" value in response.
         
         URL: ``http://www.reddit.com/api/subscribe/``
         
@@ -682,11 +692,12 @@ class Reddit(object):
         if not sr.startswith('t5_'):
             sr = self.subreddit(sr).name
         data = dict(action='sub', sr=sr)
-        return self.post('api', 'subscribe', data=data)
+        j = self.post('api', 'subscribe', data=data)
+        return assert_truthy(j)
     
     @_login_required
     def unsubscribe(self, sr):
-        """Login required.  Send POST to unsubscribe to a subreddit.  If ``sr`` is the name of the subreddit, a GET request is sent to retrieve the full id of the subreddit, which is necessary for this API call.  Returns :class:`requests.Response` object.
+        """Login required.  Send POST to unsubscribe to a subreddit.  If ``sr`` is the name of the subreddit, a GET request is sent to retrieve the full id of the subreddit, which is necessary for this API call.  Returns True or raises :class:`exceptions.UnexpectedResponse` if non-"truthy" value in response.
         
         URL: ``http://www.reddit.com/api/unsubscribe/``
         
@@ -695,7 +706,8 @@ class Reddit(object):
         if not sr.startswith('t5_'):
             sr = self.subreddit(sr).name
         data = dict(action='unsub', sr=sr)
-        return self.post('api', 'subscribe', data=data)
+        j = self.post('api', 'subscribe', data=data)
+        return assert_truthy(j)
     
     @_login_required
     def inbox(self, limit=None):
@@ -793,63 +805,77 @@ class Reddit(object):
     
     @_login_required
     def approve(self, id_):
-        """Login required.  Sends POST to approve a submission. Returns :class:`things.Listing` object.
+        """Login required.  Sends POST to approve a submission. Returns True or raises :class:`exceptions.UnexpectedResponse` if non-"truthy" value in response.
         
         URL: ``http://www.reddit.com/api/approve/``
         
-        :param limit: full id of submission to approve
+        :param id\_: full id of submission to approve
         """
         data = dict(id=id_)
-        return self.post('api', 'approve', data=data)
+        j = self.post('api', 'approve', data=data)
+        return assert_truthy(j)
     
     @_login_required
     def remove(self, id_):
-        """Login required.  Sends POST to remove a submission or comment. Returns :class:`things.Listing` object.
+        """Login required.  Sends POST to remove a submission or comment.  Returns True or raises :class:`exceptions.UnexpectedResponse` if non-"truthy" value in response.
         
         URL: ``http://www.reddit.com/api/remove/``
         
-        :param limit: full id of object to remove
+        :param id\_: full id of object to remove
         """
         data = dict(id=id_)
-        return self.post('api', 'remove', data=data)
+        j = self.post('api', 'remove', data=data)
+        return assert_truthy(j)
     
     @_login_required
-    def distinguish(self, id_, how):
-        """Login required.  Sends POST to distinguish a submission or comment. Returns :class:`requests.Response` object.
+    def distinguish(self, id_, how=True):
+        """Login required.  Sends POST to distinguish a submission or comment.  Returns :class:`things.Link` or :class:`things.Comment`, or raises :class:`exceptions.UnexpectedResponse` otherwise.
         
         URL: ``http://www.reddit.com/api/distinguish/``
         
         :param id\_: full id of object to distinguish
         :param how: either True, False, or 'admin'
         """
-        if how not in (True, False, 'admin'):
-            raise ValueError("how must be either True, False, or 'admin'")
-        data = dict(id=id_, how=how)
-        return self.post('api', 'distinguish', data=data)
+        if how == True:
+            h = 'yes'
+        elif how == False:
+            h = 'no'
+        elif how == 'admin':
+            h = 'admin'
+        else:
+            raise ValueError("how must be either True, False, or 'admin'") 
+        data = dict(id=id_)
+        j = self.post('api', 'distinguish', h, data=data)
+        try:
+            return self._thingify(j['json']['data']['things'][0])
+        except Exception:
+            raise UnexpectedResponse(j)
+        
     
     @_login_required
     def flairlist(self, r, limit=1000, after=None, before=None):
-        """Login required.  See https://github.com/reddit/reddit/wiki/API%3A-flairlist.  Returns :class:`requests.Response` object.
+        """Login required.  Gets flairlist for subreddit `r`.  See https://github.com/reddit/reddit/wiki/API%3A-flairlist.
         
-        URL: ``http://www.reddit.com/api/flairlist``
+        However, the wiki docs are wrong (as of 2012/5/4).  Returns :class:`things.ListBlob` of :class:`things.Blob` objects, each object being a mapping with `user`, `flair\_css\_class`, and `flair\_text` attributes.
+        
+        URL: ``http://www.reddit.com/r/<r>/api/flairlist``
         
         :param r: name of subreddit
         :param limit: max number of items to return
         :param after: full id of user to return entries after
         :param before: full id of user to return entries *before* 
         """
-        data = dict(r=r, limit=limit)
+        params = dict(limit=limit)
         if after:
-            data['after'] = after
+            params['after'] = after
         elif before:
-            data['before'] = before
-        else:
-            raise ValueError('after or before must be a non-empty string')
-        return self.post('api', 'flairlist', data=data)
+            params['before'] = before
+        b = self.get('r', r, 'api', 'flairlist', params=params)
+        return b.users
     
     @_login_required
     def flair(self, r, name, text, css_class):
-        """Login required.  See https://github.com/reddit/reddit/wiki/API%3A-flair.  Returns :class:`requests.Response` object.
+        """Login required.  Sets flair for a user.  See https://github.com/reddit/reddit/wiki/API%3A-flair.  Returns True or raises :class:`exceptions.UnexpectedResponse` if non-"truthy" value in response.
         
         URL: ``http://www.reddit.com/api/flair``
         
@@ -859,22 +885,24 @@ class Reddit(object):
         :param css_class: CSS class to assign to flair text
         """
         data = dict(r=r, name=name, text=text, css_class=css_class)
-        return self.post('api', 'flair', data=data)
+        j = self.post('api', 'flair', data=data)
+        return assert_truthy(j)
 
     @_login_required
     def flaircsv(self, r, flair_csv):
-        """Login required.  See https://github.com/reddit/reddit/wiki/API%3A-flaircsv.  Returns :class:`requests.Response` object.
+        """Login required.  Bulk sets flair for users.  See https://github.com/reddit/reddit/wiki/API%3A-flaircsv/.  Returns response JSON content as dict.
         
         URL: ``http://www.reddit.com/api/flaircsv``
         
         :param r: name of subreddit
-        :param flair_csv: csv file contents
+        :param flair_csv: csv string
         """
+        # TODO: handle the response better than just returning
         data = dict(r=r, flair_csv=flair_csv)
         return self.post('api', 'flaircsv', data=data)
     
     @_login_required
-    def contributors(self, sr):
+    def contributors(self, sr, limit=None):
         """Login required.  GETs list of contributors to subreddit ``sr``. Returns :class:`things.ListBlob` object.
         
         **NOTE**: The :class:`things.Account` objects in the returned ListBlob *only* have ``id`` and ``name`` set.  This is because that's all reddit returns.  If you need full info on each contributor, you must individually GET them using :meth:`user` or :meth:`things.Account.about`.
@@ -883,8 +911,32 @@ class Reddit(object):
         
         :param sr: name of subreddit
         """
-        userlist = self.get('r', sr, 'about', 'contributors')
+        userlist = self._limit_get('r', sr, 'about', 'contributors', limit=limit)
         return _process_userlist(userlist)
+    
+    @_login_required
+    def create_subreddit(self, name, title, description,
+                         lang='en', type='public', link_type='any',
+                         over_18=False, allow_top=True, show_media=False,
+                         domain=None):
+        data = dict(
+            name=name,
+            title=title,
+            description=description,
+            lang=lang,
+            type=type,
+            link_type=link_type
+        )
+        if over_18:
+            data['over_18'] = 'on'
+        if allow_top:
+            data['allow_top'] = 'on'
+        if show_media:
+            data['show_media'] = 'on'
+        if domain:
+            data['domain'] = domain
+        j = self.post('api', 'site_admin', data=data)
+        return assert_truthy(j)
 
 
 def connect(*args, **kwargs):
